keywords = """
keyword_function: ":"
keyword_import: "I"
keyword_package: "P"
keyword_export: "X:"
keyword_casting: ">>"
//keyword_scope: "(s)"
keyword_scope: "||"
//
range_keyword: ".."

typealias_keyword: "&&"
interface_keyword: "@@"
if_keyword: "if"
//searchable_kw: "~"
searchable_kw: "`" // `kata utk dicari`
fmus_keyword: "!!"
enum_keyword: "&"
exception_keyword: "x:"
csv_keyword: "csv:"
"""
