pair_operation = """
pair_operation:         penanda "P" pair_operation_choice
pair_operation_choice: "h" -> hello
"""

tuple_operation = """
tuple_operation:        penanda "L" tuple_operation_choice
tuple_operation_choice: "h" -> hello
"""
