peta_tipe_data = {
    "dart": {
        "char": "string",
        "integer": "number",
        "float": "number",
        "string": "string",
        "boolean": "boolean",
        "any": "any",
        "void": "void",
    },
    "js": {
        "char": "string",
        "integer": "number",
        "float": "number",
        "string": "string",
        "boolean": "boolean",
        "any": "any",
        "void": "void",
    },
    "ts": {
        "char": "string",
        "integer": "number",
        "float": "number",
        "string": "string",
        "boolean": "boolean",
        "any": "any",
        "void": "void",
    },
}
