# interface hanya berisi method, tapi utk antisipasi berbagai bhs yg berbeda2
# kita samakan dg class
# mari bikin interface, abstract class, dll di sini (sementara)
interface_item = f"""
interface_item: interface_keyword class_config? class_name class_content
"""
